#include "sbi.h"

#include <stdio.h>

void sbi_call(rv32_cpu_t* cpu, uint32_t extension_id, uint32_t func_id)
{
	switch(extension_id)
	{
		case SBI_EXTENSION_BASE:
		{
			switch(func_id)
			{
				case SBI_FUNCTION_GET_SPEC_VERSION:
					// Format: [31(reserved,0) 7 bits(major) 24 bits(minor)]
					// Version 2.0
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0x2000000;
					break;
				case SBI_FUNCTION_GET_IMPL_ID:
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0;
					break;
				case SBI_FUNCTION_GET_IMPL_VERSION:
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0;
					break;
				case SBI_FUNCTION_PROBE_EXTENSION:
					// For now, we say that everything is available
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 1;
					break;
				case SBI_FUNCTION_GET_MVENDOR_ID:
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0;
					break;
				case SBI_FUNCTION_GET_MARCH_ID:
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0;
					break;
				case SBI_FUNCTION_GET_MIMPL_ID:
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0;
					break;
				default:
					fprintf(stderr, "sbi_call: unknown function id 0x%x for base extension\n", func_id);
					break;
			}
			break;
		}
		case SBI_EXTENSION_TIMER:
		{
			switch(func_id)
			{
				case SBI_FUNCTION_SET_TIMER:
					cpu->regs.a0 = 0;
					cpu->regs.a1 = 0;
					break;
				default:
					fprintf(stderr, "sbi_call: unknown function id 0x%x for timer extension\n", func_id);
					break;
			}
		}
		case SBI_EXTENSION_SET_TIMER:
		{
			// TODO : Correctly implement that
			cpu->regs.a0 = 0;
			break;
		}
		case SBI_EXTENSION_LEGACY_CONSOLE_PUTCHAR:
		{
			printf("%c", cpu->regs.a0);
			cpu->regs.a0 = 0;
			break;
		}
		case SBI_EXTENSION_LEGACY_SHUTDOWN:
		{
			printf("sbi_call: shutdown, goodbye !\n");
			cpu->sim_ticks_left = 1;
			break;
		}
		default:
		{
			fprintf(stderr, "sbi_call: unknown extension id 0x%x\n", extension_id);
			break;
		}
	}
}
